# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 13:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
        ('soap_1c_api', '0003_auto_20170308_1457'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExtraCollectionRelation',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('tag_name', models.CharField(max_length=70, verbose_name='Tag name')),
                ('collection', models.OneToOneField(help_text='Link MetaContent/* tag with a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='extra_collection_relation', to='pcart_catalog.Collection', verbose_name='Collection')),
            ],
            options={
                'verbose_name': 'Extra collection relation',
                'verbose_name_plural': 'Extra collections relations',
            },
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='collection',
            field=models.OneToOneField(help_text='Add the products of specified type to a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.Collection', verbose_name='Collection'),
        ),
    ]
