# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 12:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
        ('soap_1c_api', '0002_auto_20170308_1305'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='producttyperelation',
            name='collections',
        ),
        migrations.AddField(
            model_name='producttyperelation',
            name='collection',
            field=models.OneToOneField(default=None, help_text='Add the products of specified type to a collection.', on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.Collection', verbose_name='Collections'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='product_type',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relation', to='pcart_catalog.ProductType', verbose_name='Product type'),
        ),
    ]
