# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-08 11:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('soap_1c_api', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='producttyperelation',
            name='alternate_uuid',
            field=models.UUIDField(help_text='UUID of the product category in 1C system.', unique=True, verbose_name='UUID in 1C'),
        ),
        migrations.AlterField(
            model_name='producttyperelation',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='product_type_relations', to='pcart_catalog.ProductType', verbose_name='Product type'),
        ),
    ]
