# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-09 09:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('soap_1c_api', '0004_auto_20170308_1514'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productimagedownloadlink',
            name='image',
        ),
        migrations.DeleteModel(
            name='ProductImageDownloadLink',
        ),
    ]
